/*
 * Copyright 2014 Sam Elsharif
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package info.msxlaunchers.openmsx.launcher.ui.presenter;

import info.msxlaunchers.openmsx.launcher.data.settings.constants.Language;
import info.msxlaunchers.openmsx.launcher.persistence.game.DatabaseNotFoundException;
import info.msxlaunchers.openmsx.launcher.persistence.game.GamePersister;
import info.msxlaunchers.openmsx.launcher.ui.view.DatabaseManagerView;

import java.io.IOException;
import java.util.Objects;
import java.util.Set;

import com.google.inject.Inject;

/**
 * Implementation of <code>DatabaseManagerPresenter</code>
 * 
 * @since v1.3
 * @author Sam Elsharif
 *
 */
final class DatabaseManagerPresenterImpl implements DatabaseManagerPresenter
{
	private final DatabaseManagerView view;
	private final MainPresenter mainPresenter;
	private final GamePersister gamePersister;

	//model
	private Language currentLanguage = null;
	private boolean currentRightToLeft = false;

	@Inject
	DatabaseManagerPresenterImpl( DatabaseManagerView view,	MainPresenter launcherPresenter, GamePersister gamePersister )
	{
		this.view = Objects.requireNonNull( view );
		this.mainPresenter = Objects.requireNonNull( launcherPresenter );
		this.gamePersister = Objects.requireNonNull( gamePersister );
	}

	/* (non-Javadoc)
	 * @see info.msxlaunchers.openmsx.launcher.ui.presenter.SettingsPresenter#onRequestSettingsScreen(info.msxlaunchers.openmsx.launcher.data.settings.Settings, java.util.Set, info.msxlaunchers.openmsx.launcher.data.settings.constants.Language, java.lang.String, boolean)
	 */
	@Override
	public void onRequestDatabaseManagerScreen( Set<String> databases, Language currentLanguage, boolean currentRightToLeft )
	{
		this.currentLanguage = currentLanguage;
		this.currentRightToLeft = currentRightToLeft;

		view.displayScreen( currentLanguage, currentRightToLeft, databases );
	}

	/* (non-Javadoc)
	 * @see info.msxlaunchers.openmsx.launcher.ui.presenter.DatabaseManagerPresenter#onRequestDeleteDatabase(java.lang.String)
	 */
	@Override
	public void onRequestDeleteDatabase( String database ) throws LauncherException
	{
		try
		{
			gamePersister.deleteDatabase( database );
			mainPresenter.onRequestDeleteDatabaseAction( database );
		}
		catch ( DatabaseNotFoundException dnfe )
		{
			throw new LauncherException( LauncherExceptionCode.ERR_DATABASE_NOT_FOUND, database );
		}
		catch ( IOException ioe )
		{
			throw new LauncherException( LauncherExceptionCode.ERR_IO );
		}
	}

	/* (non-Javadoc)
	 * @see info.msxlaunchers.openmsx.launcher.ui.presenter.DatabaseManagerPresenter#onRequestRenameDatabaseScreen(java.lang.String)
	 */
	@Override
	public String onRequestRenameDatabaseScreen( String database ) throws LauncherException
	{
		return view.displayRenameDatabaseScreen( database, currentLanguage, currentRightToLeft );
	}

	/* (non-Javadoc)
	 * @see info.msxlaunchers.openmsx.launcher.ui.presenter.DatabaseManagerPresenter#onRequestRenameDatabaseAction(java.lang.String, java.lang.String)
	 */
	@Override
	public void onRequestRenameDatabaseAction( String oldDatabase, String newDatabase ) throws LauncherException
	{
		try
		{
			gamePersister.renameDatabase( oldDatabase, newDatabase );
			mainPresenter.onRequestRenameDatabaseAction( oldDatabase, newDatabase );
		}
		catch ( DatabaseNotFoundException dnfe )
		{
			throw new LauncherException( LauncherExceptionCode.ERR_DATABASE_NOT_FOUND, oldDatabase );
		}
		catch ( IOException ioe )
		{
			throw new LauncherException( LauncherExceptionCode.ERR_IO );
		}
	}
}
